//=============================================================================
// CardChoser.js
//=============================================================================

/*:
 * @plugindesc Hime card choser
 * @author Hime
 *
 * @param Variable 1
 * @desc Card 1 variable.
 * @type number
 * @default 1
 *
 * @param Variable 2
 * @desc Card 2 variable.
 * @type number
 * @default 2
 *
 * @param Variable 3
 * @desc Card 3 variable.
 * @type number
 * @default 3
 *
 * @param Variable 4
 * @desc Card 4 variable.
 * @type number
 * @default 4
 *
 * @param Variable 5
 * @desc Card 5 variable.
 * @type number
 * @default 5
 *
 * @param Card start
 * @desc Card start (inclusive).
 * @type number
 * @default 601
 *
 * @param Card end
 * @desc Card end (inclusive).
 * @type number
 * @default 720
 *
 * @help
 *
 * Plugin Command:
 *   CardChoser open        # open card choser
 *   CardChoser resetCards  # reset selected cards in inventory
 *
 * Script Command:
 *   CardChoser.rank(1)     # check that all selected cards have a rarity of 1
 */

var parameters = PluginManager.parameters('AnimatedBar')
var CardChoser = {
    vars: [
        Number(parameters['Variable 1']) || 1,
        Number(parameters['Variable 2']) || 2,
        Number(parameters['Variable 3']) || 3,
        Number(parameters['Variable 4']) || 4,
        Number(parameters['Variable 5']) || 5
    ],
    cardStart: Number(parameters['Card start']) || 601,
    cardEnd: Number(parameters['Card end']) || 720,
    rank: function(rarity) {
        return CardChoser.vars.every(function(varId) {
            return $dataItems[$gameVariables.value(varId)] && +$dataItems[$gameVariables.value(varId)].meta.gachaRank === rarity
        })
    },
    hasCards: function() {
        return CardChoser.vars.every(function(varId) {
            return $dataItems[$gameVariables.value(varId)]
        })
    }
}

;(function() {
    'use strict'

    //=============================================================================
    // Scene_CardChoser
    //=============================================================================
    function Scene_CardChoser() {
        this.initialize.apply(this, arguments)
    }

    Scene_CardChoser.prototype = Object.create(Scene_MenuBase.prototype)
    Scene_CardChoser.prototype.constructor = Scene_CardChoser

    Scene_CardChoser.prototype.initialize = function() {
        Scene_MenuBase.prototype.initialize.call(this)
        this._inc = 0
        this._queue = []
    }

    Scene_CardChoser.prototype.create = function() {
        Scene_MenuBase.prototype.create.call(this)
        this.createChoseImageCommandWindow()
        this.createConfirmWindow()
    }

    Scene_CardChoser.prototype.createChoseImageCommandWindow = function() {
        this._cardChoserWindow = new Window_CardChoser(0, 0)
        this._cardChoserWindow.setHandler('ok', this.onChoseCard.bind(this))
        this._cardChoserWindow.setHandler('cancel', this.onCancelCard.bind(this))
        this._cardChoserWindow.activate()
        this._cardChoserWindow.select(0)
        this._cardChoserWindow.refresh()
        this.addWindow(this._cardChoserWindow)
    }

    Scene_CardChoser.prototype.createConfirmWindow = function() {
        this._confirmWindow = new Window_Confirm()
        this._confirmWindow.openess = 0
        this._confirmWindow.setHandler('yes', this.onConfirm.bind(this))
        this._confirmWindow.setHandler('no', this.onConfirmCancel.bind(this))
        this.addWindow(this._confirmWindow)
    }

    Scene_CardChoser.prototype.onChoseCard = function() {
        var item = this._cardChoserWindow.item()
        this._queue.push(item)
        this._cardChoserWindow.selectCard(item, true)
        $gameVariables.setValue(CardChoser.vars[this._inc], item.id)
        this._inc++
        if (this._inc < 5) {
            this._cardChoserWindow.activate()
        } else {
            this._confirmWindow.open()
            this._confirmWindow.activate()
        }
    }

    Scene_CardChoser.prototype.onCancelCard = function() {
        if (this._inc === 0) {
            this.popScene()
            return
        }
        var item = this._queue.pop()
        this._cardChoserWindow.selectCard(item, false)
        $gameVariables.setValue(CardChoser.vars[this._inc], null)
        this._inc--
        this._cardChoserWindow.activate()
    }

    Scene_CardChoser.prototype.onConfirm = function() {
        this.popScene()
    }

    Scene_CardChoser.prototype.onConfirmCancel = function() {
        this._confirmWindow.close()
        this._confirmWindow.deactivate()
        this._cardChoserWindow.activate()
    }

    //=============================================================================
    // Window_CardChoser
    //=============================================================================
    function Window_Confirm() {
        this.initialize.apply(this, arguments)
    }

    Window_Confirm.prototype = Object.create(Window_Command.prototype)
    Window_Confirm.prototype.constructor = Window_Confirm

    Window_Confirm.prototype.initialize = function() {
        Window_Command.prototype.initialize.call(this, 0, 0)
        this.openness = 0
        this.updatePlacement()
    }

    Window_Confirm.prototype.updatePlacement = function() {
        this.x = (Graphics.boxWidth - this.windowWidth()) / 2
        this.y = (Graphics.boxHeight - this.windowHeight()) / 2
    }

    Window_Confirm.prototype.makeCommandList = function() {
        this.addCommand('Confirm', 'yes')
        this.addCommand('Cancel', 'no')
    }

    //=============================================================================
    // Window_CardChoser
    //=============================================================================
    function Window_CardChoser() {
        this.initialize.apply(this, arguments)
    }

    Window_CardChoser.prototype = Object.create(Window_ItemList.prototype)
    Window_CardChoser.prototype.constructor = Window_CardChoser

    Window_CardChoser.prototype.initialize = function(x, y) {
        this._selectedCards = {}
        Window_ItemList.prototype.initialize.call(this, x, y, this.windowWidth(), this.windowHeight())
    }

    Window_CardChoser.prototype.windowWidth = function() {
        return Graphics.boxWidth
    }

    Window_CardChoser.prototype.windowHeight = function() {
        return Graphics.boxHeight
    }

    Window_CardChoser.prototype.includes = function(item) {
        return item && item.id >= CardChoser.cardStart && item.id <= CardChoser.cardEnd
    }

    Window_CardChoser.prototype.maxCols = function() {
        return 4
    }

    Window_CardChoser.prototype.isSelected = function(item) {
        return this._selectedCards[item.id] > 0
    }

    Window_CardChoser.prototype.selectCard = function(item, selected) {
        this._selectedCards[item.id] = this._selectedCards[item.id] || 0
        if (selected) {
            this._selectedCards[item.id]++
            $gameParty.gainItem(item, -1)
        } else {
            this._selectedCards[item.id]--
            $gameParty.gainItem(item, 1)
        }
        var index = this._data.indexOf(item)
        if (index > -1) {
            this.drawItem(index)
        }
    }

    Window_CardChoser.prototype.isEnabled = function(item) {
        return $gameParty.hasItem(item)
    }

    Window_CardChoser.prototype.drawItem = function(index) {
        this.clearItem(index)
        var item = this._data[index]
        if (item && this.isSelected(item)) {
            this.changeTextColor(this.textColor(2))
        } else {
            this.resetTextColor()
        }
        Window_ItemList.prototype.drawItem.call(this, index)
        this.resetTextColor()
    }

    Window_CardChoser.prototype.drawItemName = function(item, x, y, width) {
        width = width || 312
        if (item) {
            var iconBoxWidth = Window_Base._iconWidth + 4
            this.drawIcon(item.iconIndex, x + 2, y + 2)
            this.drawText(item.name, x + iconBoxWidth, y, width - iconBoxWidth)
        }
    }

    //=============================================================================
    // Game Interpreter
    //=============================================================================
    var Game_Interpreter_pluginCommand = Game_Interpreter.prototype.pluginCommand
    Game_Interpreter.prototype.pluginCommand = function(command, args) {
        Game_Interpreter_pluginCommand.call(this, command, args)
        if (command === 'CardChoser') {
            switch (args[0]) {
                case 'open':
                    CardChoser.vars.forEach(function(varId) {
                        $gameVariables.setValue(varId, 0)
                    })
                    SceneManager.push(Scene_CardChoser)
                    break
                case 'resetCards':
                    CardChoser.vars.forEach(function(varId) {
                        $gameParty.gainItem($dataItems[$gameVariables.value(varId)], 1)
                    })
                    break
            }
        }
    }
})()
