/*jslint vars: true, nomen: true, plusplus: true, eqeq: true */
/*globals console */
/*globals SceneManager, Utils, PluginManager, Graphics */
//=============================================================================
// RandomLoadingImage.js
//=============================================================================

/*:
 * @plugindesc Pivert Random loading image system.
 * @author Pivert
 *
 * @param Image directory
 * @desc Directory for loading images
 * @default system
 *
 * @param Image prefix
 * @desc Prefix for loading images
 * @default Loading
 *
 * @param Image count
 * @desc Amount of images to use
 * @default 3
 *
 */
(function () {

    'use strict';

    var parameters = PluginManager.parameters('RandomLoadingImage');
    var imgDirectory = String(parameters['Image directory'] || 'system');
    var imgPrefix = String(parameters['Image prefix'] || 'Loading');
    var imgCount = Number(parameters['Image count'] || 3);

    Graphics.endLoading = function () {
        this._clearUpperCanvas();
        this._upperCanvas.style.opacity = 0;

        // Change src image after loading
        var r = Math.randomInt(imgCount);
        var src = ['img', '/', imgDirectory, '/', imgPrefix, r, '.png'].join('');
        Graphics.setLoadingImage(src);
    };

    SceneManager.initGraphics = function () {
        var type = this.preferableRendererType();
        Graphics.initialize(this._screenWidth, this._screenHeight, type);
        Graphics.boxWidth = this._boxWidth;
        Graphics.boxHeight = this._boxHeight;
        Graphics.setLoadingImage('img/system/Loading.png');
        if (Utils.isOptionValid('showfps')) {
            Graphics.showFps();
        }
        if (type === 'webgl') {
            this.checkWebGL();
        }

        // Set first loading img
        var r = Math.randomInt(imgCount);
        var src = ['img', '/', imgDirectory, '/', imgPrefix, r, '.png'].join('');
        Graphics.setLoadingImage(src);
    };
}());
