//=============================================================================
// SceneViewer.js
//=============================================================================

/*:
 * @plugindesc Pivert Scene Viewer.
 * @author Pivert
 *
 * @param Image directory
 * @desc Image directory.
 * @default img/scene-viewer
 *
 * @param Watch variable
 * @desc Variable used to trigger watch event
 * @default 2
 *
 * @param First video ID
 * @desc ID of the first visible video (inclusive)
 * @default 700
 *
 * @param Last video ID
 * @desc ID of the last visible video (inclusive)
 * @default 900
 *
 */

(function () {
    'use strict';

    var parameters = PluginManager.parameters('SceneViewer');
    var imgDir = parameters['Image directory'] || 'img/scene-viewer';
    var watchVariable = Number(parameters['Watch variable']) || 2;
    var switchListStart = Number(parameters['First video ID']) || 700;
    var switchListEnd = Number(parameters['Last video ID']) || 900;

    //=============================================================================
    // Window_SceneViewer
    //=============================================================================
    function Window_SceneViewer() {
        this.initialize.apply(this, arguments);
    }

    Window_SceneViewer.prototype = Object.create(Window_ItemList.prototype);
    Window_SceneViewer.prototype.constructor = Window_SceneViewer;

    Window_SceneViewer.prototype.initialize = function (x, y, width, height) {
        Window_ItemList.prototype.initialize.call(this, x, y, width, height);
    };

    Window_SceneViewer._iconWidth = 128;
    Window_SceneViewer._iconHeight = 96;

    Window_SceneViewer.prototype.includes = function (scene) {
        return scene && $gameSwitches.value(scene.id);
    };

    Window_SceneViewer.prototype.isEnabled = function () {
        return true;
    };

    Window_SceneViewer.prototype.itemHeight = function () {
        return 176;
    };

    Window_SceneViewer.prototype.maxCols = function () {
        return 5;
    };

    Window_SceneViewer.prototype.makeItemList = function () {
        this._data = $dataScenes.filter(function (scene) {
            return this.includes(scene);
        }, this);
    };

    Window_SceneViewer.prototype.drawItem = function (index) {
        var scene = this._data[index];
        if (scene) {
            var rect = this.itemRect(index);
            var x = rect.x;
            var y = rect.y + 4;
            this.contents.blt(scene.bitmap, 0, 0, scene.bitmap.width, scene.bitmap.height, x + (rect.width - scene.bitmap.width) / 2, y)
            var iconBoxHeight = scene.bitmap.height + 4;
            var words = $dataSystem.switches[scene.id].split(' ');
            var index = Math.round(words.length / 2);
            var firstLine = words.slice(0, index);
            var secondLine = words.slice(index);
            this.drawText(firstLine.join(' '), x, y + iconBoxHeight, rect.width, 'center');
            this.drawText(secondLine.join(' '), x, y + iconBoxHeight + 32, rect.width, 'center');
        }
    };

    //=============================================================================
    // Scene_SceneViewer
    //=============================================================================
    function Scene_SceneViewer() {
        this.initialize.apply(this, arguments);
    }

    Scene_SceneViewer.prototype = Object.create(Scene_MenuBase.prototype);
    Scene_SceneViewer.prototype.constructor = Scene_SceneViewer;

    Scene_SceneViewer.prototype.initialize = function () {
        Scene_MenuBase.prototype.initialize.call(this);
    };

    Scene_SceneViewer.prototype.create = function () {
        Scene_MenuBase.prototype.create.call(this);
        this.createSellWindow();
    };

    Scene_SceneViewer.prototype.createSellWindow = function () {
        this._watchWindow = new Window_SceneViewer(0, 0, Graphics.boxWidth, Graphics.boxHeight);
        this._watchWindow.setHandler('ok', this.onWatchOk.bind(this));
        this._watchWindow.setHandler('cancel', this.popScene.bind(this));
        this.addWindow(this._watchWindow);
        this._watchWindow.refresh();
        this._watchWindow.activate();
        this._watchWindow.select(0);
    };

    Scene_SceneViewer.prototype.onWatchOk = function () {
        this._item = this._watchWindow.item();
        $gameVariables.setValue(watchVariable, this._item.id);
        SceneManager.pop();
    };

    //=============================================================================
    // Scene_Boot
    //=============================================================================
    var Scene_Boot_create = Scene_Boot.prototype.create;
    Scene_Boot.prototype.create = function () {
        window['$dataScenes'] = [];
        for (var i = switchListStart; i <= switchListEnd; i++) {
            window['$dataScenes'].push({
                id: i,
                img: i,
                bitmap: ImageManager.loadBitmap(`${imgDir}/`, i)
            });
        }
        Scene_Boot_create.call(this);
    };

    //=============================================================================
    // Game Interpreter
    //=============================================================================
    var Game_Interpreter_pluginCommand = Game_Interpreter.prototype.pluginCommand;
    Game_Interpreter.prototype.pluginCommand = function (command, args) {
        Game_Interpreter_pluginCommand.call(this, command, args);
        if (command === 'SceneViewer' && args[0] === 'open') {
            SceneManager.push(Scene_SceneViewer);
        }
    };
})();
