//=============================================================================
// StatusWindow.js
//=============================================================================

/*:
 * @plugindesc StatusWindow
 * @author Pivert
 *
 * @param Max stat
 * @desc Max stat.
 * @default 50
 *
 * @param Displayed actors ids
 * @desc IDs of displayed actors.
 * @default 1,2
 *
 * @param Images directory
 * @desc Images directory.
 * @default img/status-window
 *
 * @param Title color
 * @desc Title color (number or string).
 * @default 13
 *
 * @param Title font size
 * @desc Title font size.
 * @default 38
 *
 * @param Actors window title
 * @desc Actors window title.
 * @default Status
 *
 * @param Completion window title
 * @desc Completion window title.
 * @default Game completion
 *
 * @help This plugin does not provide plugin commands.
 */

;(function() {
    'use strict'

    var parameters = PluginManager.parameters('StatusWindow')
    var maxStat = Number(parameters['Max stat'] || 50)
    var actorsIds = (parameters['Displayed actors ids'] || '1,2').split(/\s*,\s*/)
    var imgDir = parameters['Images directory'] || 'img/status-window'
    var titleColor = parameters['Title color'] || 13
    var titleFontSize = Number(parameters['Title font size'] || 38)
    var actorsWindowTitle = parameters['Actors window title'] || 'Status'
    var completionWindowTitle = parameters['Completion window title'] || 'Game completion'
    var actors = []
	var actorsBitmap = {}

    //=============================================================================
    // Scene_Menu
    //=============================================================================
    var Scene_Menu_create = Scene_Menu.prototype.create
    Scene_Menu.prototype.create = function() {
        Scene_Menu_create.call(this)
        this._windowLayer.removeChild(this._statusWindow)
    }

    Scene_Menu.prototype.commandPersonal = function() {
        SceneManager.push(Scene_Status)
    }

    //=============================================================================
    // Scene_Status
    //=============================================================================
    Scene_Status.prototype.create = function() {
        Scene_MenuBase.prototype.create.call(this)

        this._actorsStatusWindow = new Window_ActorStatus()
        this.addWindow(this._actorsStatusWindow)

        this._globalStatusWindow = new Window_GlobalStatus()
        this._globalStatusWindow.setHandler('cancel', this.popScene.bind(this))
        this.addWindow(this._globalStatusWindow)
    }

    Scene_Status.prototype.refreshActor = function() {}

    Scene_Status.prototype.onActorChange = function() {}

    //=============================================================================
    // Window_GlobalStatus
    //=============================================================================
    function Window_GlobalStatus() {
        this.initialize.apply(this, arguments)
    }

    Window_GlobalStatus.prototype = Object.create(Window_Selectable.prototype)
    Window_GlobalStatus.prototype.constructor = Window_GlobalStatus

    Window_GlobalStatus.prototype.initialize = function() {
        Window_Selectable.prototype.initialize.call(this, 0, Graphics.boxHeight / 3 + 15, Graphics.boxWidth, (2 * Graphics.boxHeight) / 3 - 15)
        this.draw()
        this.activate()
    }

    Window_GlobalStatus.prototype.draw = function() {
        this.contents.clear()

        let width = this.width
        let y = 10
        let x = 0
        let yStart = this._drawStatusTitle(completionWindowTitle, x, y, width, 'center')
        yStart += 40
        $dataStatus.forEach(function(items, index, cols) {
            width = this.width / cols.length
            x = index * width
            y = yStart
            items.forEach(function(item) {
                if (!item.trigger || $gameSwitches.value(item.trigger)) {
                    y += this.drawItem(item, x, y, width - this.padding * 2)
                }
            }, this)
        }, this)
    }

    Window_GlobalStatus.prototype.drawItem = function(item, x, y, width) {
        let variant = 0
        let align = 'left'
        let color = this.normalColor()

        if (item.style) {
            color = item.style.color ? !isNaN(parseInt(item.style.color)) ? this.textColor(item.style.color) : item.style.color : color
            variant = item.style.sizeVariant || variant
            align = item.style.align || align
        }

        this.contents.fontSize = this.standardFontSize() + variant
        this.changeTextColor(color)
        this.drawText(item.name, x, y, width, align)
        this.resetTextColor()

        this.contents.fontSize = this.standardFontSize() + variant

        if (item.value) {
            let value = `Invalid item type: ${item.value.type}`
            if (item.value.type === 'variable') {
                value = $gameVariables.value(item.value.id)
            } else if (item.value.type === 'switch') {
                value = $gameSwitches.value(item.value.id) ? 'Yes' : 'No'
            } else if (item.value.type === 'const') {
                switch (item.value.id) {
                    case 'gold':
                        value = $gameParty.gold()
                        break
                    case 'time':
                        var time = $gameSystem.playtime()
                        var hours = Math.floor(time / 3600)
                        time -= hours * 3600
                        var minutes = Math.floor(time / 60)
                        time -= minutes * 60
                        var seconds = Math.floor(time)
                        value = `${hours.padZero(2)}:${minutes.padZero(2)}:${seconds.padZero(2)}`
                        break
                    case 'card':
                        value = 0
                        for (var i = item.value.intervalStart; i <= item.value.intervalEnd; i++) {
                            if ($gameParty.hasItem($dataItems[i])) {
                                value++
                            }
                        }
                        break
                    default:
                        value = `Invalid const id: ${item.value.id}`
                }
            }
            if (typeof item.value.mult !== 'undefined') {
                value *= item.value.mult
            }
            if (typeof item.value.total !== 'undefined') {
                value = `${value} / ${item.value.total}`
            }
            this.drawText(value, x, y, width, 'right')
        }

        this.contents.fontSize = this.standardFontSize()
        return this.lineHeight() + variant
    }

    //=============================================================================
    // Window_ActorStatus
    //=============================================================================
    function Window_ActorStatus() {
        this.initialize.apply(this, arguments)
    }

    Window_ActorStatus.prototype = Object.create(Window_Base.prototype)
    Window_ActorStatus.prototype.constructor = Window_ActorStatus

    Window_ActorStatus.prototype.initialize = function() {
        Window_Base.prototype.initialize.call(this, 0, 0, Graphics.boxWidth, Graphics.boxHeight / 3 + 15)
        this.draw()
    }

    Window_ActorStatus.prototype.statNameWidth = function() {
        return (
            (this._statNameWidth ||
                (this._statNameWidth = this.textWidth(
                    TextManager.param(
                        [2, 3, 4, 5, 6, 7].reduce(function(acc, curr) {
                            return TextManager.param(curr).length > TextManager.param(acc).length ? curr : acc
                        })
                    )
                ))) + 8
        )
    }

    Window_ActorStatus.prototype.draw = function() {
        this.contents.clear()

        let width = this.width
        let y = 0
        let x = 0
        let yStart = this._drawStatusTitle(actorsWindowTitle, x, y, width, 'center')
        yStart += 40
        actors.forEach(function(actor, index, actors) {
            ImageManager.reserveFace(actor.faceName())
            width = this.width / actors.length
            x = index * width
            y = yStart
            this._drawFace(actor, x, y)
            this._drawParameters(actor, x + Window_Base._faceWidth + 10, y, width - this.padding * 2 - Window_Base._faceWidth - 10)
        }, this)
    }

    Window_ActorStatus.prototype._drawFace = function(actor, x, y) {
        var bitmap = actorsBitmap[actor._actorId]
        this.contents.blt(bitmap, 0, 0, bitmap.width, bitmap.height, x, y + 6)
    }

    Window_ActorStatus.prototype._drawParameters = function(actor, x, y, width) {
        for (var i = 2; i < 8; i++) {
            this.drawStatName(x, y, i)
            this.drawStatGauge(actor, x, y, i, width)
            this.drawStatValue(actor, x, y, i, width)

            y += this.lineHeight()
        }
    }

    Window_ActorStatus.prototype.drawStatName = function(x, y, i) {
        this.changeTextColor(this.systemColor())
        this.drawText(TextManager.param(i), x, y, this.statNameWidth())
        this.resetTextColor()
    }

    Window_ActorStatus.prototype.drawStatGauge = function(actor, x, y, i, width) {
        var value = Math.min(actor.param(i), maxStat) / maxStat
        this.drawGauge(x + this.statNameWidth(), y - 4, width - this.statNameWidth(), value, this.color1(i), this.color2(i))
    }

    Window_ActorStatus.prototype.drawStatValue = function(actor, x, y, i, width) {
        var value = Math.min(actor.param(i), maxStat)
        this.drawText(value, x + this.statNameWidth(), y, width - this.statNameWidth(), 'right')
    }

    Window_ActorStatus.prototype.color1 = function(i) {
        return this.textColor([null, null, 11, 12, 18, 13, 30, 14][i])
    }

    Window_ActorStatus.prototype.color2 = function(i) {
        return this.textColor([null, null, 3, 4, 10, 5, 31, 17][i])
    }

    //=============================================================================
    // Window_Base
    //=============================================================================
    Window_Base.prototype._drawStatusTitle = function(text, x, y, width, align) {
        this.contents.fontSize = titleFontSize
        if (!isNaN(parseInt(titleColor))) {
            titleColor = this.textColor(titleColor)
        }
        this.changeTextColor(titleColor)
        this.drawText(text, x, y, width, align)
        this.contents.fontSize = this.standardFontSize()
        this.resetTextColor()
        return this.lineHeight() + titleFontSize - this.standardFontSize()
    }

    //=============================================================================
    // Scene_Boot
    //=============================================================================
    var Scene_Boot_create = Scene_Boot.prototype.create
    Scene_Boot.prototype.create = function() {
        DataManager._databaseFiles.push({ name: '$dataStatus', src: 'StatusWindow.json' })
        Scene_Boot_create.call(this)
    }

    var DataManager_extractSaveContents = DataManager.extractSaveContents
    DataManager.extractSaveContents = function(contents) {
        DataManager_extractSaveContents.call(this, contents)
        actors = actorsIds.map(function(id) {
			actorsBitmap[id] = ImageManager.reserveBitmap(imgDir + '/', id)
            return $gameActors.actor(id)
        })
    }

    var DataManager_createGameObjects = DataManager.createGameObjects
    DataManager.createGameObjects = function() {
        DataManager_createGameObjects.call(this)
        actors = actorsIds.map(function(id) {
            actorsBitmap[id] = ImageManager.reserveBitmap(imgDir + '/', id)
            return $gameActors.actor(id)
        })
    }
})()
