/*jslint vars: true, nomen: true, plusplus: true, eqeq: true */
/*globals console*/
/*globals Window_Base, Window_MenuStatus, Window_MenuCommand, Window_Selectable, Window_Gold, Window_Status*/
/*globals Scene_Menu, SceneManager, Scene_MenuBase, Scene_Skill, Scene_Equip, Scene_Status*/
/*globals PluginManager, Graphics, Bitmap, ImageManager, TouchInput, TextManager*/
/*globals $gamePlayer, $gameParty */
//=============================================================================
// RingMenu.js
//=============================================================================

/*:
 * @plugindesc Status window
 * @author Pivert
 *
 * @param Max stat
 * @desc Max stat.
 * @default 50
 *
 * @help This plugin does not provide plugin commands.
 */

(function () {
    "use strict";

    var parameters = PluginManager.parameters('Status window');
    var maxStat = Number(parameters['Max stat'] || 50);

    var descriptions;

    //=============================================================================
    // Scene menu
    //=============================================================================
    var Scene_Menu_create = Scene_Menu.prototype.create;
    Scene_Menu.prototype.create = function () {
        Scene_Menu_create.call(this);
        this._windowLayer.removeChild(this._statusWindow);
    };

    //=============================================================================
    // Scene menu
    //=============================================================================
    Scene_Menu.prototype.start = function () {
        Scene_MenuBase.prototype.start.call(this);
    };

    var Scene_Menu_createCommandWindow = Scene_Menu.prototype.createCommandWindow;
    Scene_Menu.prototype.createCommandWindow = function () {
        Scene_Menu_createCommandWindow.call(this);
        this._commandWindow.setHandler('skill', this.commandSkill.bind(this));
        this._commandWindow.setHandler('equip', this.commandEquip.bind(this));
        this._commandWindow.setHandler('status', this.commandStatus.bind(this));
    };

    Scene_Menu.prototype.commandSkill = function () { };

    Scene_Menu.prototype.commandEquip = function () { };

    Scene_Menu.prototype.commandStatus = function () {
        SceneManager.push(Scene_Status);
    };

    Scene_Menu.prototype.commandFormation = function () { };

    Scene_Menu.prototype.onPersonalOk = function () {
        SceneManager.push(Scene_Status);
    };

    //=============================================================================
    // Window menu status
    //=============================================================================
    function Window_RingMenuStatus() {
        this.initialize.apply(this, arguments);
    }

    Window_RingMenuStatus.prototype = Object.create(Window_MenuStatus.prototype);
    Window_RingMenuStatus.prototype.constructor = Window_RingMenuStatus;

    Window_RingMenuStatus.prototype.create = function () {
        Window_MenuStatus.prototype.create.apply(this);
    };

    Window_RingMenuStatus.prototype.itemWidth = function () {
        return 144;
    };

    Window_RingMenuStatus.prototype.itemHeight = function () {
        return 144;
    };

    Window_RingMenuStatus.prototype.lineHeight = function () {
        return 144;
    };

    Window_RingMenuStatus.prototype.initialize = function () {
        var width = this.itemWidth() + 36;
        var height = this.fittingHeight(2);
        Window_Selectable.prototype.initialize.call(this, Graphics.boxWidth / 4 - width, Graphics.boxHeight / 4, width, height);
        this._formationMode = false;
        this._pendingIndex = -1;
        this.loadImages();
        this.refresh();
    };

    Window_RingMenuStatus.prototype.processOk = function () {
        Window_Selectable.prototype.processOk.call(this);
        $gameParty.setMenuActor($gameParty.members()[this.index()]);
    };

    Window_RingMenuStatus.prototype.drawItemStatus = function (index) { };

    //=============================================================================
    // Window scene status
    //=============================================================================
    function Scene_Status() {
        this.initialize.apply(this, arguments);
    }

    Scene_Status.prototype = Object.create(Scene_MenuBase.prototype);
    Scene_Status.prototype.constructor = Scene_Status;

    Scene_Status.prototype.initialize = function () {
        Scene_MenuBase.prototype.initialize.call(this);
    };

    Scene_Status.prototype.create = function () {
        Scene_MenuBase.prototype.create.call(this);

        this._statusWindow = new Window_Status();
        this._statusWindow.setHandler('pagedown', this.nextActor.bind(this));
        this._statusWindow.setHandler('pageup', this.previousActor.bind(this));

        this._actorWindow = new Window_RingMenuStatus();
        this._actorWindow.setHandler('ok', this.onPersonalOk.bind(this));
        this._actorWindow.setHandler('cancel', this.popScene.bind(this));
        this._actorWindow.select(0);

        var index = this._actorWindow.index();
        this._actor = $gameParty.members()[index];
        this._actorWindow.activate();

        this.addWindow(this._actorWindow);
        this.addWindow(this._statusWindow);
        this.refreshActor();
    };

    Scene_Status.prototype.onPersonalOk = function () {
        var index = this._actorWindow.index();
        this._actor = $gameParty.members()[index];
        this._actorWindow.selectLast();
        this._actorWindow.activate();
        this.refreshActor();
    };

    Scene_Status.prototype.refreshActor = function () {
        var actor = this.actor();
        this._statusWindow.setActor(actor);
    };

    Scene_Status.prototype.onActorChange = function () {
        this.refreshActor();
    };

    //=============================================================================
    // Window_Status
    //=============================================================================
    Window_Status.prototype.initialize = function () {
        var width = Graphics.boxWidth / 2;
        var height = Graphics.boxHeight / 2;
        Window_Selectable.prototype.initialize.call(this, Graphics.boxWidth / 4, Graphics.boxHeight / 4, width, height);
        this.refresh();
        this.activate();
    };

    Window_Status.prototype.refresh = function () {
        this.contents.clear();
        if (this._actor) {
            var lineHeight = this.lineHeight();
            this.drawBlock1(0);
            this.drawBlock2(lineHeight);
            this.drawHorzLine(lineHeight * 4);
            this.drawBlock3(lineHeight * 5);
        }
    };

    Window_Status.prototype.drawBlock1 = function (y) {
        this.drawActorFace(this._actor, 12, y);
        this.drawActorName(this._actor, 180, y, 250);
    };

    function drawMultineText(window_status, text, x, y, width) {

        if (!text) {
            return;
        }

        var words = text.split(' ');

        var currentLine = '';

        var i = 0;
        for (i = 0; i < words.length; i++) {

            var word = words[i];
            if (window_status.textWidth(currentLine + word) < width) {
                currentLine += word + ' ';
            } else {
                window_status.drawText(currentLine, x, y, width);
                currentLine = word + ' ';
                y += 28;
            }
        }

        if (currentLine.length) {
            window_status.drawText(currentLine, x, y, width);
        }
    }

    Window_Status.prototype.drawBlock2 = function (y) {
        this.contents.fontSize = 18;
        var width = this.width - 180 - this.padding * 2;
        drawMultineText(this, descriptions[this._actor.name()], 180, y, width);
        this.resetFontSettings();
    };

    Window_Status.prototype.drawBlock3 = function (y) {
        this.drawParameters(48, y);
    };

    Window_Status.prototype.drawParameters = function (x, y) {
        var lineHeight = this.lineHeight();
        var colors1 = [11, 12, 18, 13, 30, 14];
        var colors2 = [3, 4, 10, 5, 31, 17];
        var i;
        for (i = 0; i < 6; i++) {
            var paramId = i + 2;
            var y2 = y + lineHeight * i;

            var value = this._actor.param(paramId);
            value = value > maxStat ? maxStat : value;
            var w = 200;

            var width = this.width - (this.padding + 48) * 2 - w;
            var color1 = this.textColor(colors1[i]);
            var color2 = this.textColor(colors2[i]);
            this.drawGauge(x + w, y2 + 10, width, 20, value / maxStat, color1, color2);
            this.changeTextColor(this.systemColor());
            this.drawText(TextManager.param(paramId), x, y2, w - 10);
            this.drawText(value, x + w + width - this.textWidth(value), y2, w - 10);
        }
    };

    Window_Status.prototype.drawGauge = function (x, y, width, height, rate, color1, color2) {
        var fillW = Math.floor(width * rate);
        this.contents.fillRect(x, y, width, height, this.gaugeBackColor());
        this.contents.gradientFillRect(x, y, fillW, height, color1, color2);
    };

    var loadDescriptions = function () {

        var xhr = new XMLHttpRequest();
        xhr.overrideMimeType('application/json');
        xhr.open('GET', 'data/Descriptions.json');

        xhr.onload = function () {
            if (xhr.status < 400) {
                try {
                    descriptions = JSON.parse(xhr.responseText);
                } catch (e) {
                    console.error('Could not load script');
                    console.error(e);
                }
            } else {
                console.error('File data/Descriptions.json not found');
            }
        };

        xhr.onerror = function (e) {
            console.error('Error while fetching data/Descriptions.json');
            console.error(e);
        };

        xhr.send();
    };

    loadDescriptions();
}());
